[PATCH] hurd: readv: Get rid of alloca
Joe Simmons-Talbott
josimmon@redhat.com
Fri Jun 16 18:29:05 GMT 2023
Replace alloca with a scratch_buffer to avoid potential stack overflows.
Checked on i686-gnu and x86_64-linux-gnu
---
sysdeps/posix/readv.c | 33 ++++++++++++---------------------
1 file changed, 12 insertions(+), 21 deletions(-)
diff --git a/sysdeps/posix/readv.c b/sysdeps/posix/readv.c
index 91208e9894..bd7379ecea 100644
--- a/sysdeps/posix/readv.c
+++ b/sysdeps/posix/readv.c
@@ -19,18 +19,12 @@
#include <unistd.h>
#include <string.h>
#include <limits.h>
+#include <scratch_buffer.h>
#include <stdbool.h>
#include <sys/param.h>
#include <sys/uio.h>
#include <errno.h>
-
-static void
-ifree (char **ptrp)
-{
- free (*ptrp);
-}
-
/* Read data from file descriptor FD, and put the result in the
buffers described by VECTOR, which is a vector of COUNT 'struct iovec's.
The buffers are filled in the order specified.
@@ -52,20 +46,15 @@ __readv (int fd, const struct iovec *vector, int count)
bytes += vector[i].iov_len;
}
- /* Allocate a temporary buffer to hold the data. We should normally
- use alloca since it's faster and does not require synchronization
- with other threads. But we cannot if the amount of memory
- required is too large. */
- char *buffer;
- char *malloced_buffer __attribute__ ((__cleanup__ (ifree))) = NULL;
- if (__libc_use_alloca (bytes))
- buffer = (char *) __alloca (bytes);
- else
- {
- malloced_buffer = buffer = (char *) malloc (bytes);
- if (buffer == NULL)
- return -1;
- }
+ /* Allocate a temporary buffer to hold the data. Use a scratch_buffer
+ since it's faster for small buffer sizes but can handle larger
+ allocations as well. */
+
+ struct scratch_buffer buf;
+ scratch_buffer_init (&buf);
+ if (!scratch_buffer_set_array_size (&buf, 1, bytes))
+ return -1;
+ char *buffer = buf.data;
/* Read the data. */
ssize_t bytes_read = __read (fd, buffer, bytes);
@@ -86,6 +75,8 @@ __readv (int fd, const struct iovec *vector, int count)
break;
}
+ scratch_buffer_free (&buf);
+
return bytes_read;
}
libc_hidden_def (__readv)
--
2.39.2
More information about the Libc-alpha
mailing list